%-------------------------------------------------------------------------
% function result=IMOLS(y,x,d,kern,band,alpha)
% 
% Function to compute the "first and second step" OLS regression of the IM-OLS
% estimator for nonlinear, linear in parameters cointegrating relationships: 
%               y = d'*delta + x'*beta + u (*)
%  Step 1 Partial Sum - with linear augmentation: 
%           Sy = Sd'*delta + Sx'*beta + x'*gamma + Su       (1)      
%  Step 2: On the version above - augmentation by Z 
%     Sy = Sd'*delta + Sx'*beta + x'*gamma + Z'*lambda + u    (2)
% Additionally, significance t-tests based on (1) are computed
% -------------------------------------------------------------------------
% Input     y ... Dependent variable in R^T (mandatory)
%
%           x ... Integrated regresors in R^{T \times dim(x)} (mandatory)
%
%           d ... Deterministic components 
%             - matrix in R^{T x dim(d)}
%             - or integer: (d = 0 indicating an intercept, d = 1
%               intercept and linear trend, d = 2 intercept, linear trend and
%               quadratic trend, etc.)          
%
%           kern  ... Specifies chosen kernel function:
%             tr ... Truncated
%             ba ... Bartlett
%             pa ... Parzen
%             bo ... Bohman
%             da ... Daniell
%             qs ... Quadratic Spectral
%
%           band ... specifies bandwith chosen, integer in the range 1,..,T
%           or use one selection rule from:
%             'And' ... data dependent rule according to Andrews (1991)
%             'NW'  ... data dependent rule according to Newey & West (1987)
%             'NWT' ... sample size dependent rule of thumb
%
%           alpha ... Significance level for t-test
%
% Default: d = 0, kern = 'ba', band = 'And', alpha = 0.05
%
% Output:   theta_hat ... [delta',beta',gamma']' coefficient vector reg (1)
%           Fitted ... fitted values \hat y
%   
%           Omega_udotv ... estimator for Omega_udotv (based on [\hat u,v],
%           with \hat u denoting OLS residuals from (1))
%           Vhat ... estimator of covariance matrix without Omega_udotv
%           (The conditional covariance matrix of theta_hat is given by
%           Omega_udotv*Vhat)
%
%           t_theta ... t-values for the coefficients in (1)
%           t_test ... test decisions for hypothesis theta_i = 0
%           (1=rejection)
%           pvalue ... corresponding pvalues
%
%           theta_hat2 ... t-values for the coefficients in (2)
%           b ... the fixed-b corresponding to the bandwidth
%           Omega_udotv_star ... estimator for Omega_udotv (based on the
%           OLS residuals from (2))
%           t_theta_fb ... corresponding t-value.
%
% External functions: Z_regressors, Vhat_NEW
%
% TJV and MW, October 2009
% Modification by RK, April 2017 
%------------------------------------------------------------------------- 
function result = IMOLS(y,x,d,kern,band,alpha)



if nargin < 2
    error(message('Arguments missing'));
end

if nargin < 3
    d = 0;
end

if nargin < 4
   kern = 'ba';
end

if nargin < 5
    band = 'And';    
end

if nargin < 6
    alpha = 0.05;
end

% Sizes of variables:
  [T,n] = size(y);
  [~,m] = size(x);

  if isscalar(d) == 1
      d = poly_regressors(T,d);
  end
    
  [~,q] = size(d);
  
% Generate Partial Summed Regressors:
  Sy = cumsum(y);
  Sx = cumsum(x);
  Sd = cumsum(d);

% Generation of regressor matrix corr. to (1):
  Xmat = [Sd Sx x]; 
  
  % Regression (1) itself:    
    % Coefficient vector:
    theta_hat = Xmat\Sy;
    result.theta_hat = theta_hat;

    % Fitted Values:
    result.Fitted = [d x]*theta_hat(1:q+m);  

% Calculation of long-run covariance:
    % first step: get OLS-residuals:
    theta_ols = [d x]\y;
    u_ols = y - [d x]*theta_ols;
    varin = [u_ols(2:end),diff(x)];        

    % determine bandwidth for long-run covariance estimation:
    if isequal(band,'And');
        bandw = And_HAC91(varin,kern);
    elseif isequal(band,'NW');
        bandw = bwNW(varin,kern,0,[]);
    elseif isequal(band,'NWT');
        bandw = floor(4*(T/100)^(2/9));
    else
        bandw = band;
    end

    % use [u_ols,delta(x)] do calculate long-run variance:
    Omega = lr_var(varin,kern,bandw,0);
    Omega_uu = Omega(1:n,1:n);
    Omega_uv = Omega(1:n,n+1:n+m);
    Omega_vu = Omega_uv';
    Omega_vv = Omega(n+1:n+m,n+1:n+m);        
    result.Omega_udotv = Omega_uu - Omega_uv*(Omega_vv\Omega_vu);

% Compute Conditional Covariance of IM-OLS estimator:
result.Vhat = Vhat_NEW(Xmat);          
varmat = result.Omega_udotv*result.Vhat;

% Compute t-values:
t_theta_pre = theta_hat./sqrt(diag(varmat));
result.t_theta = t_theta_pre(1:q+m);

% Test results:
critval = -sqrt(2).*erfcinv(2*(1-alpha/2));
result.t_test = abs(result.t_theta) > critval;

% P-value:
result.pvalue = 1 - (0.5*erfc(-result.t_theta./sqrt(2)));


% Regression (2) itself:
% Generation of Z-regressors (based on summed regressors):
Z_reg = Z_regressors(Xmat);
X2mat = [Xmat Z_reg];

% Coefficient vector:
theta_hat2 = X2mat\Sy;
result.theta_hat2 = theta_hat2;

% Residual vector: 
u_hat2 = Sy - X2mat*theta_hat2;  

% Calculation of long-run covariance:   

    % determine the fixed-b for long-run covariance estimation:
    if isequal(band,'And');
        bandw = And_HAC91(diff(u_hat2),kern);
    elseif isequal(band,'NW');
        bandw = bwNW(diff(u_hat2),kern,0,[]);
    elseif isequal(band,'NWT');
        bandw = floor(4*(T/100)^(2/9));
    else
        bandw = band;
    end
    result.b = bandw/T;

    % use [u_ols,delta(x)] do calculate long-run variance:
    result.Omega_udotv_star = lr_var(diff(u_hat2),kern,bandw,0);

% Compute Conditional Covariance of IM-OLS estimator:     
varmat_star = result.Omega_udotv_star*result.Vhat;

% Compute t-values:
t_theta_fb_pre = theta_hat./sqrt(diag(varmat_star));
result.t_theta_fb = t_theta_fb_pre(1:q+m);





