%-------------------------------------------------------------------------
% function [test_decis] = t_test_2sided_fb(testval,kern,b,p,k,alphavec);
%
% Function to evaluate whether 2-sided t-test is rejected at percentiles alphavec.
% 
% Input: testval ... Value of test statistic (vector in R^{nr. of tests
%                    \times 1})
%           kern ... Kernel function (usual abbreviation)
%              b ... Value of fixed-b parameter 0.02,0.04,...,1
%              p ... Order of deterministic polynomial (0 = constant)
%              k ... Number of integrated regressors
%       alphavec ... Vector in R^{numAlpha \times 1} with critical values
%                    considered in set: 2 times - 0.01, 0.025, 0.05
%
% Output: 
%    test_decis ... Vector in R^{numAlpha \times 1} with test decisions
%                   
%                   (0 if no rejection and 1 if rejection)
%
% Note: This procedure loads the corresponding tables with fixed-b critical
% values which need to be available at the specified place.
%
%--------------------------------------------------------------------------
function [test_decis] = t_test_2sided_fb(testval,kern,b,p,k,alphavec)

predecis = exist([pwd,filesep,'criticalvaluesimolstests',filesep,'t_FB_',kern,'_kval_',num2str(k),'_Pord_',num2str(p),'.mat']);

if isequal(predecis,2)
% Selection of critical value table:
% Load the proper table and store it in crit_val.
eval(sprintf(['load ',pwd,filesep,'criticalvaluesimolstests',filesep,'t_FB_',kern,'_kval_',num2str(k),'_Pord_',num2str(p)]));
eval(sprintf(['crittable = t_FB_',kern,'_kval_',num2str(k),'_Pord_',num2str(p),';']));

% Select the row corresponding to the b-value:
if floor(50*b) < 1
    critrow = crittable(1,:);
else
    critrow = crittable(floor(50*b),:);
end

[numA,~] = size(alphavec);
[numTest,~] = size(testval);
test_decis = zeros(numA,numTest);

for j = 1:numA; % Loop over different alpha-values
    if alphavec(j,1) == 0.01; % Choice of critical values for spec. alpha
        vlow = critrow(1);
        vhigh = critrow(11);
    elseif alphavec(j,1) == 0.05;
        vlow = critrow(3);
        vhigh = critrow(9);
    elseif alphavec(j,1) == 0.1;
        vlow = critrow(4);
        vhigh = critrow(8);
    end;
    
    % Rejection decision:
    test_decis(j,:) = max((testval < vlow),(testval > vhigh));
    
end;

test_decis = mean(test_decis,2); % Column vector

else
    prompt = 'Please enter critical value: ';
    critval = input(prompt);
    test_decis = abs(testval) > critval;
end
